@@ -1,4 +1,4 @@
-use super::{help_view::*, story_view, text_view::EditableTextView};
+use super::{help_view::*, story_view, text_view::EditableTextView, utils};
 use crate::prelude::*;
 
 #[derive(Copy, Clone, PartialEq, Eq)]
@@ -44,7 +44,7 @@ impl SearchView {
                     )))
                     .child(EditableTextView::new()),
             )
-            .child(story_view::get_story_main_view(vec![], client, 0).full_height());
+            .child(story_view::construct_story_main_view(vec![], client, 0).full_height());
 
         Self {
             mode: SearchViewMode::Search,
@@ -74,16 +74,17 @@ impl SearchView {
             None => return,
             Some(view) => view.get_text(),
         };
-        let sender = self.sender.clone();
-        let client = self.client.clone();
-        let by_date = self.by_date;
-        let page = self.page;
 
-        // use a `cb_sink` to notify the `Cursive` renderer to re-draw the application
-        // after successfully retrieving matched stories
-        let cb_sink = self.cb_sink.clone();
+        std::thread::spawn({
+            let sender = self.sender.clone();
+            let client = self.client.clone();
+            let by_date = self.by_date;
+            let page = self.page;
+
+            // use a `cb_sink` to notify the `Cursive` renderer to re-draw the application
+            // after successfully retrieving matched stories
+            let cb_sink = self.cb_sink.clone();
 
-        std::thread::spawn(
             move || match client.get_matched_stories(&query, by_date, page) {
                 Ok(stories) => {
                     sender
@@ -103,8 +104,8 @@ impl SearchView {
                         query, by_date, page, err
                     );
                 }
-            },
-        );
+            }
+        });
     }
 
     /// tries to update the Story View representing matched stories based on
@@ -130,7 +131,7 @@ impl SearchView {
         self.view.remove_child(1);
         let starting_id = client::SEARCH_LIMIT * self.page;
         self.view.add_child(
-            story_view::get_story_main_view(stories, self.client, starting_id).full_height(),
+            story_view::construct_story_main_view(stories, self.client, starting_id).full_height(),
         );
         // the old Story View is deleted hence losing the current focus,
         // we need to place the focus back to the new Story View
@@ -154,9 +155,7 @@ impl ViewWrapper for SearchView {
     }
 }
 
-/// Return a main view of a SearchView displaying the matched story list with a search bar.
-/// The main view of a SearchView is a View without status bar or footer.
-fn get_search_main_view(client: &'static client::HNClient, cb_sink: CbSink) -> impl View {
+fn construct_search_main_view(client: &'static client::HNClient, cb_sink: CbSink) -> impl View {
     let story_view_keymap = config::get_story_view_keymap().clone();
     let search_view_keymap = config::get_search_view_keymap().clone();
 
@@ -174,17 +173,17 @@ fn get_search_main_view(client: &'static client::HNClient, cb_sink: CbSink) -> i
                     _ => {
                         // handle editing shortcuts when in the search mode
                         let edit_keymap = config::get_edit_keymap().clone();
-                        if *e == edit_keymap.backward_delete_char.into() {
+                        if edit_keymap.backward_delete_char.has_event(e) {
                             view.del_char();
                             s.page = 0;
                             s.retrieve_matched_stories();
-                        } else if *e == edit_keymap.move_cursor_left.into() {
+                        } else if edit_keymap.move_cursor_left.has_event(e) {
                             view.move_cursor_left();
-                        } else if *e == edit_keymap.move_cursor_right.into() {
+                        } else if edit_keymap.move_cursor_right.has_event(e) {
                             view.move_cursor_right();
-                        } else if *e == edit_keymap.move_cursor_to_begin.into() {
+                        } else if edit_keymap.move_cursor_to_begin.has_event(e) {
                             view.move_cursor_to_begin();
-                        } else if *e == edit_keymap.move_cursor_to_end.into() {
+                        } else if edit_keymap.move_cursor_to_end.has_event(e) {
                             view.move_cursor_to_end();
                         } else {
                             return Some(EventResult::Ignored);
@@ -212,7 +211,8 @@ fn get_search_main_view(client: &'static client::HNClient, cb_sink: CbSink) -> i
                 Some(EventResult::Consumed(None))
             }
         })
-        // paging/filtering while in NavigationMode
+        // paging/filtering commands while in NavigationMode
+        // Those commands need to be handled differently from the story view.
         .on_pre_event_inner(story_view_keymap.toggle_sort_by_date, |s, _| match s.mode {
             SearchViewMode::Navigation => {
                 s.page = 0;
@@ -240,26 +240,27 @@ fn get_search_main_view(client: &'static client::HNClient, cb_sink: CbSink) -> i
             }
             SearchViewMode::Search => Some(EventResult::Ignored),
         })
+        .on_pre_event(config::get_global_keymap().open_help_dialog.clone(), |s| {
+            s.add_layer(SearchView::construct_on_event_help_view());
+        })
 }
 
-/// Return a view representing a SearchView that searches stories with queries
-pub fn get_search_view(client: &'static client::HNClient, cb_sink: CbSink) -> impl View {
-    let main_view = get_search_main_view(client, cb_sink);
+fn construct_search_view(client: &'static client::HNClient, cb_sink: CbSink) -> impl View {
+    let main_view = construct_search_main_view(client, cb_sink);
+
     let mut view = LinearLayout::vertical()
         .child(utils::construct_view_title_bar("Search View"))
         .child(main_view)
         .child(utils::construct_footer_view::<SearchView>());
+
     view.set_focus_index(1)
         .unwrap_or(EventResult::Consumed(None));
 
-    OnEventView::new(view).on_event(config::get_global_keymap().open_help_dialog.clone(), |s| {
-        s.add_layer(SearchView::construct_help_view());
-    })
+    view
 }
 
-/// Add a SearchView as a new layer to the main Cursive View
-pub fn add_search_view_layer(s: &mut Cursive, client: &'static client::HNClient) {
+pub fn construct_and_add_new_search_view(s: &mut Cursive, client: &'static client::HNClient) {
     let cb_sink = s.cb_sink().clone();
     s.screen_mut()
-        .add_transparent_layer(Layer::new(get_search_view(client, cb_sink)));
+        .add_transparent_layer(Layer::new(construct_search_view(client, cb_sink)));
 }
