@@ -1,9 +1,6 @@
-use super::article_view;
-use super::async_view;
-use super::comment_view;
-use super::help_view::HasHelpView;
-use super::list_view::*;
-use super::text_view;
+use super::{
+    article_view, async_view, comment_view, help_view::HasHelpView, text_view, traits::*, utils,
+};
 use crate::client::StoryNumericFilters;
 use crate::prelude::*;
 
@@ -14,47 +11,58 @@ static STORY_TAGS: [&str; 5] = ["front_page", "story", "ask_hn", "show_hn", "job
 pub struct StoryView {
     pub stories: Vec<client::Story>,
 
-    view: ScrollListView,
+    view: ScrollView<LinearLayout>,
     raw_command: String,
 }
 
 impl ViewWrapper for StoryView {
-    wrap_impl!(self.view: ScrollListView);
+    wrap_impl!(self.view: ScrollView<LinearLayout>);
 }
 
 impl StoryView {
     pub fn new(stories: Vec<client::Story>, starting_id: usize) -> Self {
-        // `max_id_width` is equal to the width of the string representation of `max_id`,
-        // which is the maximum story's id that current story view will display
-        let max_id = starting_id + stories.len() + 1;
-        let mut max_id_width = 0;
-        let mut pw = 1;
-        while pw <= max_id {
-            pw *= 10;
-            max_id_width += 1;
-        }
-
-        let view = LinearLayout::vertical()
-            .with(|s| {
-                stories.iter().enumerate().for_each(|(i, story)| {
-                    let mut story_text = StyledString::styled(
-                        format!("{1:>0$}. ", max_id_width, starting_id + i + 1),
-                        config::get_config_theme().component_style.metadata,
-                    );
-                    story_text.append(Self::get_story_text(max_id_width, story));
-                    s.add_child(text_view::TextView::new(story_text));
-                })
-            })
-            .scrollable();
         StoryView {
-            view,
+            view: Self::construct_story_view(&stories, starting_id),
             stories,
             raw_command: String::new(),
         }
     }
 
-    /// Get the description text summarizing basic information about a story
-    fn get_story_text(max_id_width: usize, story: &client::Story) -> StyledString {
+    fn construct_story_view(
+        stories: &[client::Story],
+        starting_id: usize,
+    ) -> ScrollView<LinearLayout> {
+        // Determine the maximum length of a story's ID string.
+        // This maximum length is used to align the display of the story IDs.
+        let max_id_len = {
+            let max_id = starting_id + stories.len() + 1;
+            let mut width = 0;
+            let mut pw = 1;
+            while pw <= max_id {
+                pw *= 10;
+                width += 1;
+            }
+
+            width
+        };
+
+        LinearLayout::vertical()
+            .with(|s| {
+                stories.iter().enumerate().for_each(|(i, story)| {
+                    let mut story_text = StyledString::styled(
+                        format!("{1:>0$}. ", max_id_len, starting_id + i + 1),
+                        config::get_config_theme().component_style.metadata,
+                    );
+                    story_text.append(Self::get_story_text(max_id_len, story));
+
+                    s.add_child(text_view::TextView::new(story_text));
+                })
+            })
+            .scrollable()
+    }
+
+    /// Get the text summarizing basic information about a story
+    fn get_story_text(max_id_len: usize, story: &client::Story) -> StyledString {
         let mut story_text = story.title.clone();
 
         if let Ok(url) = url::Url::parse(&story.url) {
@@ -69,36 +77,63 @@ impl StoryView {
         story_text.append_plain("\n");
 
         story_text.append_styled(
-            // left-align the story's metadata by `max_id_width+2`
+            // left-align the story's metadata by `max_id_len+2`
             // which is the width of the string "{max_story_id}. "
             format!(
                 "{:width$}{} points | by {} | {} ago | {} comments",
                 " ",
                 story.points,
                 story.author,
-                utils::get_elapsed_time_as_text(story.time),
+                crate::utils::get_elapsed_time_as_text(story.time),
                 story.num_comments,
-                width = max_id_width + 2,
+                width = max_id_len + 2,
             ),
             config::get_config_theme().component_style.metadata,
         );
         story_text
     }
 
-    inner_getters!(self.view: ScrollListView);
+    inner_getters!(self.view: ScrollView<LinearLayout>);
 }
 
-/// Return a main view of a StoryView displaying the story list.
-/// The main view of a StoryView is a View without status bar or footer.
-pub fn get_story_main_view(
+impl ListViewContainer for StoryView {
+    fn get_inner_list(&self) -> &LinearLayout {
+        self.get_inner().get_inner()
+    }
+
+    fn get_inner_list_mut(&mut self) -> &mut LinearLayout {
+        self.get_inner_mut().get_inner_mut()
+    }
+
+    fn on_set_focus_index(&mut self, old_id: usize, new_id: usize) {
+        let direction = old_id <= new_id;
+
+        // enable auto-scrolling when changing the focused index of the view
+        self.scroll(direction);
+    }
+}
+
+impl ScrollViewContainer for StoryView {
+    type ScrollInner = LinearLayout;
+
+    fn get_inner_scroll_view(&self) -> &ScrollView<LinearLayout> {
+        self.get_inner()
+    }
+
+    fn get_inner_scroll_view_mut(&mut self) -> &mut ScrollView<LinearLayout> {
+        self.get_inner_mut()
+    }
+}
+
+pub fn construct_story_main_view(
     stories: Vec<client::Story>,
     client: &'static client::HNClient,
     starting_id: usize,
 ) -> OnEventView<StoryView> {
-    let story_view_keymap = config::get_story_view_keymap().clone();
-
     let is_suffix_key =
-        |c: &Event| -> bool { *c == config::get_story_view_keymap().goto_story.clone().into() };
+        |c: &Event| -> bool { config::get_story_view_keymap().goto_story.has_event(c) };
+
+    let story_view_keymap = config::get_story_view_keymap().clone();
 
     OnEventView::new(StoryView::new(stories, starting_id))
         // number parsing
@@ -113,7 +148,10 @@ pub fn get_story_main_view(
                     }
                 }
             };
-            None
+
+            // don't allow the inner `LinearLayout` child view to handle the event
+            // because of its pre-defined `on_event` function
+            Some(EventResult::Ignored)
         })
         // story navigation shortcuts
         .on_pre_event_inner(story_view_keymap.prev_story, |s, _| {
@@ -134,13 +172,13 @@ pub fn get_story_main_view(
             // so it can be cloned without greatly affecting performance
             let story = s.stories[id].clone();
             Some(EventResult::with_cb({
-                move |s| comment_view::add_comment_view_layer(s, client, &story, false)
+                move |s| comment_view::construct_and_add_new_comment_view(s, client, &story, false)
             }))
         })
         // open external link shortcuts
         .on_pre_event_inner(story_view_keymap.open_article_in_browser, move |s, _| {
             let id = s.get_focus_index();
-            utils::open_url_in_browser(&s.stories[id].url);
+            utils::open_url_in_browser(s.stories[id].get_url().as_ref());
             Some(EventResult::Consumed(None))
         })
         .on_pre_event_inner(
@@ -150,7 +188,7 @@ pub fn get_story_main_view(
                 let url = s.stories[id].url.clone();
                 if !url.is_empty() {
                     Some(EventResult::with_cb({
-                        move |s| article_view::add_article_view_layer(s, &url)
+                        move |s| article_view::construct_and_add_new_article_view(s, &url)
                     }))
                 } else {
                     Some(EventResult::Consumed(None))
@@ -158,8 +196,7 @@ pub fn get_story_main_view(
             },
         )
         .on_pre_event_inner(story_view_keymap.open_story_in_browser, move |s, _| {
-            let id = s.stories[s.get_focus_index()].id;
-            let url = format!("{}/item?id={}", client::HN_HOST_URL, id);
+            let url = s.stories[s.get_focus_index()].story_url();
             utils::open_url_in_browser(&url);
             Some(EventResult::Consumed(None))
         })
@@ -181,6 +218,7 @@ pub fn get_story_main_view(
                 Err(_) => None,
             }
         })
+        .on_scroll_events()
 }
 
 fn get_story_view_title_bar(tag: &'static str) -> impl View {
@@ -216,8 +254,8 @@ fn get_story_view_title_bar(tag: &'static str) -> impl View {
     )
 }
 
-/// Return a StoryView given a story list and the view description
-pub fn get_story_view(
+/// Construct a story view given a list of stories.
+pub fn construct_story_view(
     stories: Vec<client::Story>,
     client: &'static client::HNClient,
     tag: &'static str,
@@ -226,7 +264,7 @@ pub fn get_story_view(
     numeric_filters: client::StoryNumericFilters,
 ) -> impl View {
     let starting_id = client::STORY_LIMIT * page;
-    let main_view = get_story_main_view(stories, client, starting_id).full_height();
+    let main_view = construct_story_main_view(stories, client, starting_id).full_height();
 
     let mut view = LinearLayout::vertical()
         .child(get_story_view_title_bar(tag))
@@ -242,20 +280,24 @@ pub fn get_story_view(
 
     let story_view_keymap = config::get_story_view_keymap().clone();
 
+    // Because we re-use the story main view to construct a search view,
+    // some of the story keymaps need to be handled here instead of by the main view like
+    // for comment views or article views.
+
     OnEventView::new(view)
         .on_pre_event(config::get_global_keymap().open_help_dialog.clone(), |s| {
-            s.add_layer(StoryView::construct_help_view())
+            s.add_layer(StoryView::construct_on_event_help_view())
         })
-        .on_event(story_view_keymap.toggle_sort_by_date, move |s| {
+        .on_pre_event(story_view_keymap.toggle_sort_by_date, move |s| {
             // disable "search_by_date" for front_page stories
             if tag == "front_page" {
                 return;
             }
-            add_story_view_layer(s, client, tag, !by_date, 0, numeric_filters, true);
+            construct_and_add_new_story_view(s, client, tag, !by_date, 0, numeric_filters, true);
         })
         // story tag navigation
-        .on_event(story_view_keymap.next_story_tag, move |s| {
-            add_story_view_layer(
+        .on_pre_event(story_view_keymap.next_story_tag, move |s| {
+            construct_and_add_new_story_view(
                 s,
                 client,
                 STORY_TAGS[(current_tag_pos + 1) % STORY_TAGS.len()],
@@ -265,8 +307,8 @@ pub fn get_story_view(
                 false,
             );
         })
-        .on_event(story_view_keymap.prev_story_tag, move |s| {
-            add_story_view_layer(
+        .on_pre_event(story_view_keymap.prev_story_tag, move |s| {
+            construct_and_add_new_story_view(
                 s,
                 client,
                 STORY_TAGS[(current_tag_pos + STORY_TAGS.len() - 1) % STORY_TAGS.len()],
@@ -277,18 +319,34 @@ pub fn get_story_view(
             );
         })
         // paging
-        .on_event(story_view_keymap.prev_page, move |s| {
+        .on_pre_event(story_view_keymap.prev_page, move |s| {
             if page > 0 {
-                add_story_view_layer(s, client, tag, by_date, page - 1, numeric_filters, true);
+                construct_and_add_new_story_view(
+                    s,
+                    client,
+                    tag,
+                    by_date,
+                    page - 1,
+                    numeric_filters,
+                    true,
+                );
             }
         })
-        .on_event(story_view_keymap.next_page, move |s| {
-            add_story_view_layer(s, client, tag, by_date, page + 1, numeric_filters, true);
+        .on_pre_event(story_view_keymap.next_page, move |s| {
+            construct_and_add_new_story_view(
+                s,
+                client,
+                tag,
+                by_date,
+                page + 1,
+                numeric_filters,
+                true,
+            );
         })
 }
 
-/// Add a StoryView as a new layer to the main Cursive View
-pub fn add_story_view_layer(
+/// Retrieve a list of stories satisfying some conditions and construct a story view displaying them.
+pub fn construct_and_add_new_story_view(
     s: &mut Cursive,
     client: &'static client::HNClient,
     tag: &'static str,
@@ -298,7 +356,7 @@ pub fn add_story_view_layer(
     pop_layer: bool,
 ) {
     let async_view =
-        async_view::get_story_view_async(s, client, tag, by_date, page, numeric_filters);
+        async_view::construct_story_view_async(s, client, tag, by_date, page, numeric_filters);
     if pop_layer {
         s.pop_layer();
     }
